/*
 *	netcam.h
 *
 *	Include file for handling network cameras.
 *
 *	This code was inspired by the original netcam.c module
 *	written by Jeroen Vreeken and enhanced by several Motion
 *	project contributors, particularly Angel Carpintero and
 *	Christopher Price.
 *	
 *	Copyright 2005, William M. Brack
 *	This software is distributed under the GNU Public license
 *	Version 2.  See also the file 'COPYING'.
 */
#ifndef _INCLUDE_NETCAM_H
#define _INCLUDE_NETCAM_H

#include <jpeglib.h>
#include <setjmp.h>
#include <sys/socket.h>
#include <sys/types.h>

#include "netcam_wget.h"        /* needed for struct rbuf */

#define NETCAM_NBUFF 2          /* Number of image buffers */
#define NETCAM_BUFFSIZE 4096    /* Initial size reserved for a JPEG
                                   image.  If expansion is required,
                                   this value is also used for the
                                   amount to increase. */

/*
 * struct url_t is used when parsing the user-supplied URL, as well as
 * when attempting to connect to the netcam.
 */
struct url_t {
    char *service;
    char *userpass;
    char *host;
    int port;
    char *path;
};

/*
 * struct netcam_context contains all the structures and other data
 * for an individual netcam.
 */
struct netcam_context {
	struct context *cnt;    /* pointer to parent motion
	                           context structure */

	int finish;             /* flag to break the camera-
	                           handling thread out of it's
	                           infinite loop in emergency */

	int threadnr;           /* motion's thread number for
	                           the camera-handling thread
	                           (if required).  Used for
	                           error reporting */

	pthread_t thread_id;    /* thread i.d. for a camera-
	                           handling thread (if required).
	                           Not currently used, but may be
	                           useful in the future */

	pthread_mutex_t mutex;  /* global mutex, protecting all
	                           the elements of the structure.
	                           Used to control shared access
	                           when a camera-handling thread
	                           is present. */

	struct url_t url;       /* structure containing the parsed
	                           URL to be used when connecting
	                           to the camera */

	char *connect_request;  /* contains the complete string
	                           required for connection to the
	                           camera */

	int sock;               /* fd for the camera's socket.
	                           Note that this value is also
	                           present within the struct
	                           rbuf *response. */

	struct rbuf *response;  /* this structure (defined in the
	                           netcam_wget module) contains
	                           the context for an HTTP
	                           connection.  Note that this
	                           structure includes a large
	                           buffer for the HTTP data */

	struct netcam_caps {    /* netcam capabilities: */
		unsigned char streaming;        /*  1 - supported       */
		unsigned char content_length;   /*  0 - unsupported     */
	} caps;

	char *boundary;         /* 'boundary' string when used to
	                           separate mjpeg images */

	int boundary_length;    /* string length of the boundary
	                           string */

	int latest;             /* multiple image buffers are used.
	                           This variable shows which buffer
	                           contains the latest completed
	                           frame received from the camera */

	int next;               /* index of next available buff */

	struct netcam_image {
		struct netcam_image_buffer {
			char *ptr;
			int content_length;
			size_t size;	/* total allocated size */
			size_t used;	/* bytes already used */
		} buffer;
	} image[NETCAM_NBUFF];

	int width;              /* info for decompression */
	int height;
	unsigned char *video_image;    /* working buff for decompression */
	int video_image_size;          /* size of decompression buffer */

	struct jpeg_error_mgr jerr;
	jmp_buf setjmp_buffer;
};


/*
 * Declare prototypes for our external entry points
 */
/*     Within netcam_jpeg.c    */
void netcam_proc_jpeg (struct netcam_context *);
void netcam_get_dimensions (struct netcam_context *);
/*     Within netcam.c        */
int netcam_start (struct context *);
unsigned char *netcam_next (struct context *, unsigned char *);
void netcam_cleanup (struct netcam_context *);

#endif
